<?php

namespace Drupal\domain_lang;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\domain\DomainNegotiatorInterface;
use Drupal\language\ConfigurableLanguageManagerInterface;
use Drupal\language\LanguageNegotiatorInterface;
use Drupal\language\Plugin\LanguageNegotiation\LanguageNegotiationUI;

/**
 * Domain language handling.
 */
class DomainLangHandler implements DomainLangHandlerInterface {

  /**
   * The domain negotiator.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface
   */
  protected $domainNegotiator;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The language negotiation method plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $negotiatorManager;

  /**
   * The language manager.
   *
   * @var \Drupal\language\ConfigurableLanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The language negotiator.
   *
   * @var \Drupal\language\LanguageNegotiatorInterface
   */
  protected $languageNegotiator;

  /**
   * The domain context of the request.
   *
   * @var \Drupal\domain\DomainInterface $domain
   */
  protected $domain;

  /**
   * Language types config name.
   *
   * @var string
   */
  protected $languageTypesConfig;

  /**
   * Constructs a new class object.
   *
   * @param \Drupal\domain\DomainNegotiatorInterface $domain_negotiator
   *   The domain negotiator service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $negotiator_manager
   *   The language negotiation methods plugin manager.
   * @param \Drupal\language\ConfigurableLanguageManagerInterface $language_manager
   *    The language manager.
   * @param \Drupal\language\LanguageNegotiatorInterface $language_negotiator
   *   The language negotiation methods manager.
   */
  public function __construct(DomainNegotiatorInterface $domain_negotiator, ConfigFactoryInterface $config_factory, PluginManagerInterface $negotiator_manager, ConfigurableLanguageManagerInterface $language_manager, LanguageNegotiatorInterface $language_negotiator) {
    $this->domainNegotiator = $domain_negotiator;
    $this->configFactory = $config_factory;
    $this->negotiatorManager = $negotiator_manager;
    $this->languageManager = $language_manager;
    $this->languageNegotiator = $language_negotiator;
    $this->domain = $this->domainNegotiator->getActiveDomain();
    $this->languageTypesConfig = $this->getEditableConfig('language.types');
  }

  /**
   * Return editable config object by config name.
   *
   * @param string $config_name
   *   The name of the config object.
   *
   * @return \Drupal\Core\Config\Config
   *   Editable config object.
   */
  private function getEditableConfig($config_name) {
    return $this->configFactory->getEditable($this->getDomainConfigName($config_name));
  }

  /**
   * {@inheritdoc}
   */
  public function getDomainConfigName($config_name) {
    return 'domain.config.' . $this->domain->id() . '.' . $config_name;
  }

  /**
   * {@inheritdoc}
   */
  public function updateConfigItem($config_name, $key, $value) {
    $this->updateConfigItems($config_name, [$key => $value]);
  }

  /**
   * {@inheritdoc}
   */
  public function updateConfigItems($config_name, array $items) {
    $config = $this->getEditableConfig($config_name);

    foreach ($items as $key => $value) {
      $config->set($key, $value);
    }

    $config->save();
  }

  /**
   * {@inheritdoc}
   */
  public function replaceConfig($config_name, array $data) {
    $config = $this->getEditableConfig($config_name);
    $config->setData($data);
    $config->save();
  }

  /**
   * {@inheritdoc}
   */
  public function removeConfigItem($config_name, $key) {
    $this->removeConfigItems($config_name, [$key]);
  }

  /**
   * {@inheritdoc}
   */
  public function removeConfigItems($config_name, array $keys) {
    $config = $this->getEditableConfig($config_name);

    foreach ($keys as $key) {
      $config->clear($key);
    }

    $config->save();
  }

  /**
   * {@inheritdoc}
   */
  public function updateLanguageTypes($key, $value) {
    $this->languageTypesConfig->set($key, $value)->save();
  }

  /**
   * {@inheritdoc}
   */
  public function updateConfiguration(array $types) {
    // Ensure that we are getting the defined language negotiation information.
    // An invocation of \Drupal\Core\Extension\ModuleInstaller::install() or
    // \Drupal\Core\Extension\ModuleInstaller::uninstall() could invalidate the
    // cached information.
    $this->negotiatorManager->clearCachedDefinitions();
    $this->languageManager->reset();

    $language_types = array();
    $language_types_info = $this->languageManager->getDefinedLanguageTypesInfo();
    $method_definitions = $this->languageNegotiator->getNegotiationMethods();

    foreach ($language_types_info as $type => $info) {
      $configurable = in_array($type, $types);

      // The default language negotiation settings, if available, are stored in
      // $info['fixed'].
      $has_default_settings = !empty($info['fixed']);
      // Check whether the language type is unlocked. Only the status of
      // unlocked language types can be toggled between configurable and
      // non-configurable.
      if (empty($info['locked'])) {
        if (!$configurable && !$has_default_settings) {
          // If we have an unlocked non-configurable language type without
          // default language negotiation settings, we use the values
          // negotiated for the interface language which, should always be
          // available.
          $method_weights = array(LanguageNegotiationUI::METHOD_ID);
          $method_weights = array_flip($method_weights);
          $this->saveConfiguration($type, $method_weights);
        }
      }
      else {
        // The language type is locked. Locked language types with default
        // settings are always considered non-configurable. In turn if default
        // settings are missing, the language type is always considered
        // configurable.

        // If the language type is locked we can just store its default language
        // negotiation settings if it has some, since it is not configurable.
        if ($has_default_settings) {
          $method_weights = array();
          // Default settings are in $info['fixed'].

          foreach ($info['fixed'] as $weight => $method_id) {
            if (isset($method_definitions[$method_id])) {
              $method_weights[$method_id] = $weight;
            }
          }
          $this->saveConfiguration($type, $method_weights);
        }
        else {
          // It was missing default settings, so force it to be configurable.
          $configurable = TRUE;
        }
      }

      // Accumulate information for each language type so it can be saved later.
      $language_types[$type] = $configurable;
    }

    // Store the language type configuration.
    $config = array(
      'configurable' => array_keys(array_filter($language_types)),
      'all' => array_keys($language_types),
    );
    $this->saveLanguageTypesConfiguration($config);
  }

  /**
   * {@inheritdoc}
   */
  public function saveLanguageTypesConfiguration(array $values) {
    if (isset($values['configurable'])) {
      $this->languageTypesConfig->set('configurable', $values['configurable']);
    }
    if (isset($values['all'])) {
      $this->languageTypesConfig->set('all', $values['all']);
    }
    $this->languageTypesConfig->save();
  }

  /**
   * {@inheritdoc}
   */
  public function saveConfiguration($type, $enabled_methods) {
    // As configurable language types might have changed, we reset the cache.
    $this->languageManager->reset();
    $definitions = $this->languageNegotiator->getNegotiationMethods();
    $default_types = $this->languageManager->getLanguageTypes();

    // Order the language negotiation method list by weight.
    asort($enabled_methods);
    foreach ($enabled_methods as $method_id => $weight) {
      if (isset($definitions[$method_id])) {
        $method = $definitions[$method_id];
        // If the language negotiation method does not express any preference
        // about types, make it available for any configurable type.
        $types = array_flip(!empty($method['types']) ? $method['types'] : $default_types);
        // Check whether the method is defined and has the right type.
        if (!isset($types[$type])) {
          unset($enabled_methods[$method_id]);
        }
      }
      else {
        unset($enabled_methods[$method_id]);
      }
    }
    $this->languageTypesConfig->set('negotiation.' . $type . '.enabled', $enabled_methods)->save();
  }

}
