<?php

namespace Drupal\domain_lang;

/**
 * Interface for Domain lang service.
 */
interface DomainLangHandlerInterface {

  /**
   * Get configuration name for this hostname.
   *
   * @param string $config_name
   *   The name of the config object.
   *
   * @return string
   *   The domain-specific config name.
   */
  public function getDomainConfigName($config_name);

  /**
   * Insert or Update config single item.
   *
   * @param string $config_name
   *   The name of the config object.
   * @param string $key
   *   Config key for update.
   * @param mixed $value
   *   Value to set.
   */
  public function updateConfigItem($config_name, $key, $value);

  /**
   * Insert or Update multiple items in requested config.
   *
   * @param string $config_name
   *   The name of the config object.
   * @param array $items
   *   Items for update.
   */
  public function updateConfigItems($config_name, array $items);

  /**
   * Replaces the data of requested configuration object.
   *
   * @param string $config_name
   *   The name of the config object.
   * @param array $data
   *   The new configuration data.
   */
  public function replaceConfig($config_name, array $data);

  /**
   * Remove item from requested config.
   *
   * @param string $config_name
   *   The name of the config object.
   * @param string $key
   *   Config key for remove.
   */
  public function removeConfigItem($config_name, $key);

  /**
   * Remove multiple items from requested config.
   *
   * @param string $config_name
   *   The name of the config object.
   * @param array $keys
   *   Keys for remove.
   */
  public function removeConfigItems($config_name, array $keys);

  /**
   * Direct update language types config for current active domain.
   *
   * @param string $key
   *   Config key for update.
   * @param mixed $value
   *   Value to set.
   */
  public function updateLanguageTypes($key, $value);

  /**
   * Updates the configuration based on the given language types.
   *
   * Stores the list of the language types along with information about their
   * configurable state. Stores the default settings if the language type is
   * not configurable.
   *
   * @param string[] $types
   *   An array of configurable language types.
   */
  public function updateConfiguration(array $types);

  /**
   * Stores language types configuration.
   *
   * @param array $values
   *   An indexed array with the following keys_
   *   - configurable: an array of configurable language type names.
   *   - all: an array of all the defined language type names.
   */
  public function saveLanguageTypesConfiguration(array $values);

  /**
   * Saves a list of language negotiation methods for a language type.
   *
   * @param string $type
   *   The language type.
   * @param int[] $enabled_methods
   *   An array of language negotiation method weights keyed by method ID.
   */
  public function saveConfiguration($type, $enabled_methods);

}
