<?php

namespace Drupal\domain_lang\EventSubscriber;

use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Url;
use Drupal\domain\DomainInterface;
use Drupal\domain\DomainLoaderInterface;
use Drupal\domain\DomainNegotiatorInterface;
use Symfony\Cmf\Component\Routing\RouteObjectInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Redirect subscriber for control language detection and selection pages.
 */
class DomainLangConfigRedirect implements EventSubscriberInterface {

  /**
   * The domain negotiator.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface $domainNegotiator
   */
  protected $domainNegotiator;

  /**
   * The domain loader.
   *
   * @var \Drupal\domain\DomainLoaderInterface $domainLoader
   */
  protected $domainLoader;

  /**
   * Currently active domain.
   *
   * @var \Drupal\domain\DomainInterface $domain
   */
  protected $domain;

  /**
   * Constructs a new class object.
   *
   * @param \Drupal\domain\DomainNegotiatorInterface $domain_negotiator
   *   The domain negotiator service.
   * @param \Drupal\domain\DomainLoaderInterface $domain_loader
   *   The domain loader service.
   */
  public function __construct(DomainNegotiatorInterface $domain_negotiator, DomainLoaderInterface $domain_loader) {
    $this->domainNegotiator = $domain_negotiator;
    $this->domainLoader = $domain_loader;
    $this->domain = $this->domainNegotiator->getActiveDomain();
  }

  /**
   * Check current request and redirect if needed.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   Current request response event.
   */
  public function checkRequest(GetResponseEvent $event) {
    $request = $event->getRequest();

    switch ($request->get(RouteObjectInterface::ROUTE_NAME)) {

      case 'domain_lang.admin':
        if ($request->get('domain') !== $this->domain->id()) {
          drupal_set_message(t('You was redirected to related domain language settings page.'));
          drupal_set_message(t('Language detection and selection can be correctly tuned ONLY on related domain.'));
          $actual_domain = $this->domainLoader->load($request->get('domain'));
          $event->setResponse(
            $this->getRedirectResponse('domain_lang.admin', $actual_domain)
          );
        }
        break;

      case 'language.negotiation':
        drupal_set_message(t('You was redirected to current active domain language settings page.'));
        drupal_set_message(t('This page should be used for currently active domain language detection and selection setup.'));
        $event->setResponse(
          $this->getRedirectResponse('domain_lang.admin', $this->domain)
        );
        break;

      case 'language.negotiation_session':
        $args = ['@type' => t('Session language detection configuration')];
        drupal_set_message(t('You was redirected to current active domain @type page.', $args));
        drupal_set_message(t('This page should be used for currently active domain @type.', $args));
        $event->setResponse(
          $this->getRedirectResponse('domain_lang.negotiation_session', $this->domain)
        );
        break;

      case 'language.negotiation_browser':
        $args = ['@type' => t('Browser language detection configuration')];
        drupal_set_message(t('You was redirected to current active domain @type page.', $args));
        drupal_set_message(t('This page should be used for currently active domain @type.', $args));
        $event->setResponse(
          $this->getRedirectResponse('domain_lang.negotiation_browser', $this->domain)
        );
        break;

      case 'language.negotiation_url':
        $args = ['@type' => t('URL language detection configuration')];
        drupal_set_message(t('You was redirected to current active domain @type page.', $args));
        drupal_set_message(t('This page should be used for currently active domain @type.', $args));
        $event->setResponse(
          $this->getRedirectResponse('domain_lang.negotiation_url', $this->domain)
        );
        break;

      case 'language.negotiation_selected':
        $args = ['@type' => t('Selected language configuration')];
        drupal_set_message(t('You was redirected to current active domain @type page.', $args));
        drupal_set_message(t('This page should be used for currently active domain @type.', $args));
        $event->setResponse(
          $this->getRedirectResponse('domain_lang.negotiation_selected', $this->domain)
        );
        break;
    }
  }

  /**
   * Return TrustedRedirectResponse for redirect to related domain page.
   *
   * @param string $route
   *   The name of the route
   * @param \Drupal\domain\DomainInterface $domain
   *   Domain object.
   *
   * @return \Drupal\Core\Routing\TrustedRedirectResponse
   *   Redirect to related domain response.
   */
  protected function getRedirectResponse($route, DomainInterface $domain) {
    $url = Url::fromRoute($route, ['domain' => $domain->id()]);
    return new TrustedRedirectResponse($domain->getScheme() . $domain->getHostname() . $url->toString());
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = array('checkRequest');
    return $events;
  }
}
